@@ -19,15 +19,52 @@ def calcCandlesSimilarity(candles0: trdb2py.trading2_pb2.Candles, candles1: trdb
     arr = []
     for i in range(len(arr0)):
         if arr1[i] == arr0[i]:
-            arr.append(1)
-        elif arr0[i] > arr1[i]:
+            arr.append(1.0)
+        elif arr0[i] > arr1[i] and arr1[i] >= 0:
             arr.append(arr1[i] / arr0[i])
-        elif arr1[i] > arr0[i]:
+        elif arr1[i] > arr0[i] and arr0[i] >= 0:
             arr.append(arr0[i] / arr1[i])
+        elif arr0[i] > arr1[i] and arr0[i] <= 0:
+            arr.append(arr0[i] / arr1[i])
+        elif arr1[i] > arr0[i] and arr1[i] <= 0:
+            arr.append(arr1[i] / arr0[i])            
+        else:
+            arr.append(-1.0)
 
     return np.mean(arr)
 
 
+def calcCandlesSimilarity_ln(candles0: trdb2py.trading2_pb2.Candles, candles1: trdb2py.trading2_pb2.Candles):
+    if len(candles0.candles) != len(candles1.candles):
+        raise ValueError
+
+    arr0 = []
+    arr1 = []
+
+    for candle in candles0.candles:
+        arr0.append(candle.close - candle.open)
+
+    for candle in candles1.candles:
+        arr1.append(candle.close - candle.open)
+
+    arr = []
+    for i in range(len(arr0)):
+        if arr1[i] == arr0[i]:
+            arr.append(np.log(1.0))
+        elif arr0[i] > arr1[i] and arr1[i] >= 0:
+            arr.append(np.log(arr1[i] / arr0[i]))
+        elif arr1[i] > arr0[i] and arr0[i] >= 0:
+            arr.append(np.log(arr0[i] / arr1[i]))
+        elif arr0[i] > arr1[i] and arr0[i] <= 0:
+            arr.append(np.log(arr0[i] / arr1[i]))
+        elif arr1[i] > arr0[i] and arr1[i] <= 0:
+            arr.append(np.log(arr1[i] / arr0[i]))
+        else:
+            arr.append(np.log(-1.0))
+
+    return np.mean(arr)    
+
+
 def calcCandlesSimilarity_LowLevel(candles0: trdb2py.trading2_pb2.Candles, candles1: trdb2py.trading2_pb2.Candles):
     if len(candles0.candles) != len(candles1.candles):
         raise ValueError
